@@ -1,6 +1,5 @@
 """Implementation of code management magic functions.
 """
-from __future__ import print_function
 #-----------------------------------------------------------------------------
 #  Copyright (c) 2012 The IPython Development Team.
 #
@@ -21,6 +20,8 @@ import re
 import sys
 import ast
 from itertools import chain
+from urllib.request import urlopen
+from urllib.parse import urlencode
 
 # Our own packages
 from IPython.core.error import TryNext, StdinNotImplementedError, UsageError
@@ -28,11 +29,10 @@ from IPython.core.macro import Macro
 from IPython.core.magic import Magics, magics_class, line_magic
 from IPython.core.oinspect import find_file, find_source_lines
 from IPython.testing.skipdoctest import skip_doctest
-from IPython.utils import py3compat
-from IPython.utils.py3compat import string_types
 from IPython.utils.contexts import preserve_keys
-from IPython.utils.path import get_py_filename, unquote_filename
-from IPython.utils.warn import warn, error
+from IPython.utils.path import get_py_filename
+from warnings import warn
+from logging import error
 from IPython.utils.text import get_text_list
 
 #-----------------------------------------------------------------------------
@@ -54,7 +54,7 @@ $""", re.VERBOSE)
 
 def extract_code_ranges(ranges_str):
     """Turn a string of range for %%load into 2-tuples of (start, stop)
-    ready to use as a slice of the content splitted by lines.
+    ready to use as a slice of the content split by lines.
 
     Examples
     --------
@@ -81,7 +81,6 @@ def extract_code_ranges(ranges_str):
         yield (start, end)
 
 
-@skip_doctest
 def extract_symbols(code, symbols):
     """
     Return a tuple  (blocks, not_found)
@@ -91,14 +90,12 @@ def extract_symbols(code, symbols):
 
     For example::
 
-        >>> code = '''a = 10
+        In [1]: code = '''a = 10
+           ...: def b(): return 42
+           ...: class A: pass'''
 
-        def b(): return 42
-
-        class A: pass'''
-
-        >>> extract_symbols(code, 'A,b,z')
-        (["class A: pass", "def b(): return 42"], ['z'])
+        In [2]: extract_symbols(code, 'A,b,z')
+        Out[2]: (['class A: pass\\n', 'def b(): return 42\\n'], ['z'])
     """
     symbols = symbols.split(',')
 
@@ -136,6 +133,36 @@ def extract_symbols(code, symbols):
 
     return blocks, not_found
 
+def strip_initial_indent(lines):
+    """For %load, strip indent from lines until finding an unindented line.
+
+    https://github.com/ipython/ipython/issues/9775
+    """
+    indent_re = re.compile(r'\s+')
+
+    it = iter(lines)
+    first_line = next(it)
+    indent_match = indent_re.match(first_line)
+
+    if indent_match:
+        # First line was indented
+        indent = indent_match.group()
+        yield first_line[len(indent):]
+
+        for line in it:
+            if line.startswith(indent):
+                yield line[len(indent):]
+            else:
+                # Less indented than the first line - stop dedenting
+                yield line
+                break
+    else:
+        yield first_line
+
+    # Pass the remaining lines through without dedenting
+    for line in it:
+        yield line
+
 
 class InteractivelyDefined(Exception):
     """Exception for interactively defined variable in magic_edit"""
@@ -147,6 +174,10 @@ class InteractivelyDefined(Exception):
 class CodeMagics(Magics):
     """Magics related to code management (loading, saving, editing, ...)."""
 
+    def __init__(self, *args, **kwargs):
+        self._knowntemps = set()
+        super(CodeMagics, self).__init__(*args, **kwargs)
+
     @line_magic
     def save(self, parameter_s=''):
         """Save a set of lines or a macro to a given filename.
@@ -182,9 +213,9 @@ class CodeMagics(Magics):
         force = 'f' in opts
         append = 'a' in opts
         mode = 'a' if append else 'w'
-        ext = u'.ipy' if raw else u'.py'
-        fname, codefrom = unquote_filename(args[0]), " ".join(args[1:])
-        if not fname.endswith((u'.py',u'.ipy')):
+        ext = '.ipy' if raw else '.py'
+        fname, codefrom = args[0], " ".join(args[1:])
+        if not fname.endswith(('.py','.ipy')):
             fname += ext
         file_exists = os.path.isfile(fname)
         if file_exists and not force and not append:
@@ -201,20 +232,19 @@ class CodeMagics(Magics):
         except (TypeError, ValueError) as e:
             print(e.args[0])
             return
-        out = py3compat.cast_unicode(cmds)
         with io.open(fname, mode, encoding="utf-8") as f:
             if not file_exists or not append:
-                f.write(u"# coding: utf-8\n")
-            f.write(out)
+                f.write("# coding: utf-8\n")
+            f.write(cmds)
             # make sure we end on a newline
-            if not out.endswith(u'\n'):
-                f.write(u'\n')
+            if not cmds.endswith('\n'):
+                f.write('\n')
         print('The following commands were written to file `%s`:' % fname)
         print(cmds)
 
     @line_magic
     def pastebin(self, parameter_s=''):
-        """Upload code to Github's Gist paste bin, returning the URL.
+        """Upload code to dpaste's paste bin, returning the URL.
 
         Usage:\\
           %pastebin [-d "Custom description"] 1-7
@@ -235,25 +265,14 @@ class CodeMagics(Magics):
             print(e.args[0])
             return
 
-        # Deferred import
-        try:
-            from urllib.request import urlopen # Py 3
-        except ImportError:
-            from urllib2 import urlopen
-        import json
-        post_data = json.dumps({
-          "description": opts.get('d', "Pasted from IPython"),
-          "public": True,
-          "files": {
-            "file1.py": {
-              "content": code
-            }
-          }
+        post_data = urlencode({
+          "title": opts.get('d', "Pasted from IPython"),
+          "syntax": "python3",
+          "content": code
         }).encode('utf-8')
 
-        response = urlopen("https://api.github.com/gists", post_data)
-        response_data = json.loads(response.read().decode('utf-8'))
-        return response_data['html_url']
+        response = urlopen("http://dpaste.com/api/v2/", post_data)
+        return response.headers.get('Location')
 
     @line_magic
     def loadpy(self, arg_s):
@@ -335,18 +354,18 @@ class CodeMagics(Magics):
             lines = contents.split('\n')
             slices = extract_code_ranges(ranges)
             contents = [lines[slice(*slc)] for slc in slices]
-            contents = '\n'.join(chain.from_iterable(contents))
+            contents = '\n'.join(strip_initial_indent(chain.from_iterable(contents)))
 
         l = len(contents)
 
-        # 200 000 is ~ 2500 full 80 caracter lines
+        # 200 000 is ~ 2500 full 80 character lines
         # so in average, more than 5000 lines
         if l > 200000 and 'y' not in opts:
             try:
                 ans = self.shell.ask_yes_no(("The text you're trying to load seems pretty big"\
                 " (%d characters). Continue (y/[N]) ?" % l), default='n' )
             except StdinNotImplementedError:
-                #asume yes if raw input not implemented
+                #assume yes if raw input not implemented
                 ans = True
 
             if ans is False :
@@ -363,7 +382,6 @@ class CodeMagics(Magics):
 
         def make_filename(arg):
             "Make a filename from the given args"
-            arg = unquote_filename(arg)
             try:
                 filename = get_py_filename(arg)
             except IOError:
@@ -410,7 +428,7 @@ class CodeMagics(Magics):
 
                     #print '*** args',args,'type',type(args)  # dbg
                     data = eval(args, shell.user_ns)
-                    if not isinstance(data, string_types):
+                    if not isinstance(data, str):
                         raise DataIsObject
 
                 except (NameError,SyntaxError):
@@ -661,6 +679,12 @@ class CodeMagics(Magics):
             # just give up.
             return
 
+        if is_temp:
+            self._knowntemps.add(filename)
+        elif (filename in self._knowntemps):
+            is_temp = True
+
+
         # do actual editing here
         print('Editing...', end=' ')
         sys.stdout.flush()
@@ -696,7 +720,8 @@ class CodeMagics(Magics):
 
         if is_temp:
             try:
-                return open(filename).read()
+                with open(filename) as f:
+                    return f.read()
             except IOError as msg:
                 if msg.filename == filename:
                     warn('File not found. Did you forget to save?')
