@@ -155,7 +155,12 @@ def qtapi_version():
     try:
         import sip
     except ImportError:
-        return
+        # as of PyQt5 5.11, sip is no longer available as a top-level
+        # module and needs to be imported from the PyQt5 namespace
+        try:
+            from PyQt5 import sip
+        except ImportError:
+            return
     try:
         return sip.getapi('QString')
     except ValueError:
@@ -217,10 +222,9 @@ def import_pyqt5():
 
     ImportErrors rasied within this function are non-recoverable
     """
-    import sip
 
     from PyQt5 import QtCore, QtSvg, QtWidgets, QtGui
-
+    
     # Alias PyQt-specific functions for PySide compatibility.
     QtCore.Signal = QtCore.pyqtSignal
     QtCore.Slot = QtCore.pyqtSlot
@@ -283,7 +287,7 @@ def load_qt(api_options):
     Raises
     ------
     ImportError, if it isn't possible to import any requested
-    bindings (either becaues they aren't installed, or because
+    bindings (either because they aren't installed, or because
     an incompatible library has already been installed)
     """
     loaders = {
