@@ -14,18 +14,6 @@ Extra capabilities for IPython
 # Imports
 #-----------------------------------------------------------------------------
 
-from IPython.lib.inputhook import (
-    enable_wx, disable_wx,
-    enable_gtk, disable_gtk,
-    enable_qt4, disable_qt4,
-    enable_tk, disable_tk,
-    enable_glut, disable_glut,
-    enable_pyglet, disable_pyglet,
-    enable_gtk3, disable_gtk3,
-    set_inputhook, clear_inputhook,
-    current_gui
-)
-
 from IPython.lib.security import passwd
 
 #-----------------------------------------------------------------------------
