@@ -86,10 +86,16 @@ class Audio(DisplayObject):
         Audio(b'RAW_WAV_DATA..)  # From bytes
         Audio(data=b'RAW_WAV_DATA..)
 
+    See Also
+    --------
+    
+    See also the ``Audio`` widgets form the ``ipywidget`` package for more flexibility and options.
+    
     """
     _read_flags = 'rb'
 
-    def __init__(self, data=None, filename=None, url=None, embed=None, rate=None, autoplay=False, normalize=True):
+    def __init__(self, data=None, filename=None, url=None, embed=None, rate=None, autoplay=False, normalize=True, *,
+                 element_id=None):
         if filename is None and url is None and data is None:
             raise ValueError("No audio data found. Expecting filename, url, or data.")
         if embed is False and url is None:
@@ -100,6 +106,7 @@ class Audio(DisplayObject):
         else:
             self.embed = True
         self.autoplay = autoplay
+        self.element_id = element_id
         super(Audio, self).__init__(data=data, url=url, filename=filename)
 
         if self.data is not None and not isinstance(self.data, bytes):
@@ -123,7 +130,6 @@ class Audio(DisplayObject):
     @staticmethod
     def _make_wav(data, rate, normalize):
         """ Transform a numpy array to a PCM bytestring """
-        import struct
         from io import BytesIO
         import wave
 
@@ -138,7 +144,7 @@ class Audio(DisplayObject):
         waveobj.setframerate(rate)
         waveobj.setsampwidth(2)
         waveobj.setcomptype('NONE','NONE')
-        waveobj.writeframes(b''.join([struct.pack('<h',x) for x in scaled]))
+        waveobj.writeframes(scaled)
         val = fp.getvalue()
         waveobj.close()
 
@@ -163,12 +169,17 @@ class Audio(DisplayObject):
         
         max_abs_value = np.max(np.abs(data))
         normalization_factor = Audio._get_normalization_factor(max_abs_value, normalize)
-        scaled = np.int16(data / normalization_factor * 32767).tolist()
-        return scaled, nchan
+        scaled = data / normalization_factor * 32767
+        return scaled.astype('<h').tostring(), nchan
 
 
     @staticmethod
     def _validate_and_normalize_without_numpy(data, normalize):
+        import array
+        import sys
+
+        data = array.array('f', data)
+
         try:
             max_abs_value = float(max([abs(x) for x in data]))
         except TypeError:
@@ -176,9 +187,11 @@ class Audio(DisplayObject):
                 'supported if numpy is not installed')
 
         normalization_factor = Audio._get_normalization_factor(max_abs_value, normalize)
-        scaled = [int(x / normalization_factor * 32767) for x in data]
+        scaled = array.array('h', [int(x / normalization_factor * 32767) for x in data])
+        if sys.byteorder == 'big':
+            scaled.byteswap()
         nchan = 1
-        return scaled, nchan
+        return scaled.tobytes(), nchan
 
     @staticmethod
     def _get_normalization_factor(max_abs_value, normalize):
@@ -198,12 +211,13 @@ class Audio(DisplayObject):
 
     def _repr_html_(self):
         src = """
-                <audio controls="controls" {autoplay}>
+                <audio {element_id} controls="controls" {autoplay}>
                     <source src="{src}" type="{type}" />
                     Your browser does not support the audio element.
                 </audio>
               """
-        return src.format(src=self.src_attr(),type=self.mimetype, autoplay=self.autoplay_attr())
+        return src.format(src=self.src_attr(), type=self.mimetype, autoplay=self.autoplay_attr(),
+                          element_id=self.element_id_attr())
 
     def src_attr(self):
         import base64
@@ -221,6 +235,12 @@ class Audio(DisplayObject):
             return 'autoplay="autoplay"'
         else:
             return ''
+    
+    def element_id_attr(self):
+        if (self.element_id):
+            return 'id="{element_id}"'.format(element_id=self.element_id)
+        else:
+            return ''
 
 class IFrame(object):
     """
@@ -486,7 +506,7 @@ class FileLinks(FileLink):
              substituted for the first and fname will be substituted for the
              second
            fp_format: string to use for formatting filepaths, must contain
-            exactly two "%s" and the dirname will be subsituted for the first
+            exactly two "%s" and the dirname will be substituted for the first
             and fname will be substituted for the second
         """
         def f(dirname, fnames, included_suffixes=None):
