@@ -1,7 +1,7 @@
 # encoding: utf-8
 
 """
-Enable wxPython to be used interacive by setting PyOS_InputHook.
+Enable wxPython to be used interactively by setting PyOS_InputHook.
 
 Authors:  Robin Dunn, Brian Granger, Ondrej Certik
 """
@@ -142,10 +142,7 @@ def inputhook_wx3():
                 # 0.01    1.5%
                 # 0.05    0.5%
                 used_time = clock() - t
-                if used_time > 5*60.0:
-                    # print 'Sleep for 5 s'  # dbg
-                    time.sleep(5.0)
-                elif used_time > 10.0:
+                if used_time > 10.0:
                     # print 'Sleep for 1 s'  # dbg
                     time.sleep(1.0)
                 elif used_time > 0.1:
