@@ -10,6 +10,7 @@ import tempfile
 import shutil
 import subprocess
 from base64 import encodebytes
+import textwrap
 
 from IPython.utils.process import find_cmd, FindCmdError
 from traitlets.config import get_config
@@ -22,7 +23,7 @@ class LaTeXTool(SingletonConfigurable):
     """An object to store configuration of the LaTeX tool."""
     def _config_default(self):
         return get_config()
-    
+
     backends = List(
         Unicode(), ["matplotlib", "dvipng"],
         help="Preferred backend to draw LaTeX math equations. "
@@ -55,7 +56,8 @@ class LaTeXTool(SingletonConfigurable):
         ).tag(config=True)
 
 
-def latex_to_png(s, encode=False, backend=None, wrap=False):
+def latex_to_png(s, encode=False, backend=None, wrap=False, color='Black',
+                 scale=1.0):
     """Render a LaTeX string to PNG.
 
     Parameters
@@ -68,6 +70,11 @@ def latex_to_png(s, encode=False, backend=None, wrap=False):
         Backend for producing PNG data.
     wrap : bool
         If true, Automatically wrap `s` as a LaTeX equation.
+    color : string
+        Foreground color name among dvipsnames, e.g. 'Maroon' or on hex RGB
+        format, e.g. '#AA20FA'.
+    scale : float
+        Scale factor for the resulting PNG.
 
     None is returned when the backend cannot be used.
 
@@ -82,15 +89,25 @@ def latex_to_png(s, encode=False, backend=None, wrap=False):
         f = latex_to_png_mpl
     elif backend == 'dvipng':
         f = latex_to_png_dvipng
+        if color.startswith('#'):
+            # Convert hex RGB color to LaTeX RGB color.
+            if len(color) == 7:
+                try:
+                    color = "RGB {}".format(" ".join([str(int(x, 16)) for x in
+                                                      textwrap.wrap(color[1:], 2)]))
+                except ValueError:
+                    raise ValueError('Invalid color specification {}.'.format(color))
+            else:
+                raise ValueError('Invalid color specification {}.'.format(color))
     else:
         raise ValueError('No such backend {0}'.format(backend))
-    bin_data = f(s, wrap)
+    bin_data = f(s, wrap, color, scale)
     if encode and bin_data:
         bin_data = encodebytes(bin_data)
     return bin_data
 
 
-def latex_to_png_mpl(s, wrap):
+def latex_to_png_mpl(s, wrap, color='Black', scale=1.0):
     try:
         from matplotlib import mathtext
         from pyparsing import ParseFatalException
@@ -105,13 +122,14 @@ def latex_to_png_mpl(s, wrap):
     try:
         mt = mathtext.MathTextParser('bitmap')
         f = BytesIO()
-        mt.to_png(f, s, fontsize=12)
+        dpi = 120*scale
+        mt.to_png(f, s, fontsize=12, dpi=dpi, color=color)
         return f.getvalue()
     except (ValueError, RuntimeError, ParseFatalException):
         return None
 
 
-def latex_to_png_dvipng(s, wrap):
+def latex_to_png_dvipng(s, wrap, color='Black', scale=1.0):
     try:
         find_cmd('latex')
         find_cmd('dvipng')
@@ -131,10 +149,11 @@ def latex_to_png_dvipng(s, wrap):
                 ["latex", "-halt-on-error", "-interaction", "batchmode", tmpfile],
                 cwd=workdir, stdout=devnull, stderr=devnull)
 
+            resolution = round(150*scale)
             subprocess.check_call(
-                ["dvipng", "-T", "tight", "-x", "1500", "-z", "9",
-                 "-bg", "transparent", "-o", outfile, dvifile], cwd=workdir,
-                stdout=devnull, stderr=devnull)
+                ["dvipng", "-T", "tight", "-D", str(resolution), "-z", "9",
+                 "-bg", "transparent", "-o", outfile, dvifile, "-fg", color],
+                 cwd=workdir, stdout=devnull, stderr=devnull)
 
         with open(outfile, "rb") as f:
             return f.read()
@@ -179,7 +198,7 @@ def genelatex(body, wrap):
         yield u'$${0}$$'.format(body)
     else:
         yield body
-    yield u'\end{document}'
+    yield u'\\end{document}'
 
 
 _data_uri_template_png = u"""<img src="data:image/png;base64,%s" alt=%s />"""
