@@ -4,7 +4,6 @@
 # Distributed under the terms of the Modified BSD License.
 
 import IPython.testing.tools as tt
-from IPython.testing.decorators import skip_without
 
 
 def test_ipython_help():
@@ -25,6 +24,5 @@ def test_locate_help():
 def test_locate_profile_help():
     tt.help_all_output_test("locate profile")
 
-@skip_without('IPython.nbformat')  # Requires jsonschema to be installed
 def test_trust_help():
     tt.help_all_output_test("trust")
